#!/usr/bin/env python
# -*- coding: UTF-8 -*-

""" Aug 2007
GUI by Ismail ari
Audio Search by Sıddıka Parlak
Sign Clustering by Marek Hruz, Oya Aran and Pavel Campr
"""

import wx
from wx import xrc
import os
import MFCD_Reader
import MFCD_Functions
import MFCD_Timers
import MFCD_Client
import MFCD_Player
import MFCD_Preferences


class MFCD(wx.App):
    
    def OnInit(self):
        # variables
        self.serviceCheckRate = 60  # in seconds
        self.client = MFCD_Client.MFCD_Client()
        self.searchResultsFolder = "searchResults" + os.sep
        self.recentSearches = []
        self.preferences = MFCD_Preferences.MFCD_Preferences()
        self.currentData = MFCD_Reader.MFCD_Data()
        self.volume = 1.0
        self.speed = 1.0
        self.leftAddition = 0 # in msec
        self.rightAddition = 0 # in msec

        self.res = xrc.XmlResource('MFCD_GUI.xrc')
        self.Init_frame()
        
        
        return True

    def Init_frame(self):
        self.frame = self.res.LoadFrame(None, 'mainFrame')
        try:
            self.frame.SetIcon(wx.Icon("images" + os.sep + "signiary.ico", wx.BITMAP_TYPE_ICO))
        finally:
            pass
        
        # to reference an xrc control
        self.mainPanel = xrc.XRCCTRL(self.frame, 'mainPanel')
        self.btnSearchLocater = xrc.XRCCTRL(self.frame, 'btnSearchLocater')
        self.treectrlSearch = xrc.XRCCTRL(self.frame, 'treectrlSearch')
        self.statusBar = xrc.XRCCTRL(self.frame, 'statusBar')
        self.stxtProperties = xrc.XRCCTRL(self.frame, 'stxtProperties')
        self.btnVideoDisplayLocater = xrc.XRCCTRL(self.frame, 'btnVideoDisplayLocater')
        self.btnPlay = xrc.XRCCTRL(self.frame, 'btnPlay')
        self.sldSeek = xrc.XRCCTRL(self.frame, 'sldSeek')
        self.btnPause = xrc.XRCCTRL(self.frame, 'btnPause')
        self.btnStop = xrc.XRCCTRL(self.frame, 'btnStop')
        self.sldVolume = xrc.XRCCTRL(self.frame, 'sldVolume')
        self.sldSpeed = xrc.XRCCTRL(self.frame, 'sldSpeed')
        self.sldLeftAddition = xrc.XRCCTRL(self.frame, 'sldLeftAddition')
        self.sldRightAddition = xrc.XRCCTRL(self.frame, 'sldRightAddition')
        self.btnOriginalVideoFolder = xrc.XRCCTRL(self.frame, 'btnOriginalVideoFolder')
        self.btnSegmentedVideoFolder = xrc.XRCCTRL(self.frame, 'btnSegmentedVideoFolder')
        self.chkBoxUseLocalVideos = xrc.XRCCTRL(self.frame, 'chkBoxUseLocalVideos')
        self.chkBoxUseLocalVideos.SetValue(self.preferences.useLocalVideos)
        self.chkBoxShowSegmented = xrc.XRCCTRL(self.frame, 'chkBoxShowSegmented')
        self.chkBoxShowSegmented.SetValue(self.preferences.showSegmented)
        
        # new added widgets
        searchSizer = self.btnSearchLocater.GetContainingSizer()
        self.searchCtrl = wx.SearchCtrl(self.mainPanel, size=(-1,-1), style=wx.TE_PROCESS_ENTER)
        searchSizer.Add(self.searchCtrl, 1, wx.ALL, 5)
        self.ResetSearchHistoryMenu()
        videoDisplaySizer = self.btnVideoDisplayLocater.GetContainingSizer()
        self.player = MFCD_Player.MFCD_Player(self.mainPanel, id=wx.ID_ANY)
        self.playerState = False
        videoDisplaySizer.Add(self.player, 0, wx.ALL, 10)

        
        # to bind an event to an xrc control
        self.frame.Bind(wx.EVT_CLOSE, self.OnQuitApplication)
        self.frame.Bind(wx.EVT_MENU, self.OnQuitApplication, id=xrc.XRCID('menuItemQuit'))
        self.frame.Bind(wx.EVT_MENU, self.OnAboutApplication, id=xrc.XRCID('menuItemAbout'))
        self.frame.Bind(wx.EVT_MENU, self.OnHelpApplication, id=xrc.XRCID('menuItemHelp'))
        self.frame.Bind(wx.EVT_TEXT_ENTER, self.OnDoSearch, self.searchCtrl)
        self.Bind(wx.EVT_TEXT, self.OnSearchTextChanged, self.searchCtrl)
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnSelChanged, self.treectrlSearch)
        self.frame.Bind(wx.EVT_BUTTON, self.OnPlay, self.btnPlay)
        self.frame.Bind(wx.EVT_SLIDER, self.OnSeek, self.sldSeek)
        self.frame.Bind(wx.EVT_BUTTON, self.OnPause, id=xrc.XRCID('btnPause'))
        self.frame.Bind(wx.EVT_BUTTON, self.OnStop, id=xrc.XRCID('btnStop'))
        self.frame.Bind(wx.EVT_SLIDER, self.OnVolumeChange, self.sldVolume)
        self.frame.Bind(wx.EVT_SLIDER, self.OnSpeedChange, self.sldSpeed)
        self.frame.Bind(wx.EVT_SLIDER, self.OnLeftAddition, self.sldLeftAddition)
        self.frame.Bind(wx.EVT_SLIDER, self.OnRightAddition, self.sldRightAddition)
        self.frame.Bind(wx.EVT_BUTTON, self.OnOriginalVideoFolderSelect, self.btnOriginalVideoFolder)
        self.frame.Bind(wx.EVT_BUTTON, self.OnSegmentedVideoFolderSelect, self.btnSegmentedVideoFolder)
        self.frame.Bind(wx.EVT_CHECKBOX, self.OnUseLocalVideosChecked, self.chkBoxUseLocalVideos)
        self.frame.Bind(wx.EVT_CHECKBOX, self.OnShowSegmentedChecked, self.chkBoxShowSegmented)
        
        # statusbar settings
        self.statusBar.SetFieldsCount(2)
        self.statusBar.SetStatusWidths([-3, -1])

        # timers
        self.frame.playerTimer = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.OnPlayerTimer)
        self.frame.playerTimer.Start(100)
        
        self.frame.serviceTimer = MFCD_Timers.serviceTimer(self.statusBar, self.client)
        self.frame.serviceTimer.Start(self.serviceCheckRate * 1000)
        self.statusBar.SetStatusText("Checking service availability...", 1)
        self.frame.serviceTimer.Notify()
        
        #self.frame.Maximize(True)
        self.frame.Show()
        self.mainPanel.GetSizer().Layout()
        self.searchCtrl.SetFocus()

    def LoadVideo(self, filename):
        if self.preferences.useLocalVideos:
            if self.preferences.showSegmented:
                filepath = self.preferences.segmentedVideosFolder + filename
            else:
                filepath = self.preferences.originalVideosFolder + filename
            if not os.path.exists(filepath):
                wx.MessageBox("File does not exist: %s. Please be sure that the folder is correct" % filepath,
                      "ERROR",
                      wx.ICON_ERROR | wx.OK)
                return
        else:
            if self.preferences.showSegmented:
                filepath = self.preferences.segmentedVideosFolderWeb + filename
            else:
                filepath = self.preferences.originalVideosFolderWeb + filename
            if not self.client.IsServiceAvailable():
                wx.MessageBox(self.frame, 'No internet service available.\n Try to use local videos instead.'
                                'Service Unavailable',
                                wx.ICON_ERROR | wx.OK)
                return
            
        if self.preferences.useLocalVideos:
            isLoaded = self.player.Load(filepath)
        else:
            isLoaded = self.player.LoadFromURI(filepath)
        if not isLoaded:
            wx.MessageBox("Unable to load %s: Unsupported format?" % filepath, "ERROR",
                          wx.ICON_ERROR | wx.OK)
        else:
            self.player.SetInitialSize()
            if self.preferences.showSegmented:
                f = open(filepath[0:-5] + ".length", "r")
                surplus = int(f.read())
                f.close
                self.currentData.starttime += surplus # surplus in the segmented videos
            self.player.InitInfo(self.currentData.starttime, self.currentData.duration, self.leftAddition, self.rightAddition, filepath)
            self.playerState = True
            self.cursor = wx.BusyCursor()
            self.info = wx.BusyInfo("Loading. Please wait...")
            self.mainPanel.Disable()
            
    def OnPlayerTimer(self, event):
        if self.playerState == True:
            self.player.SetVolume(self.volume)
            self.player.SetPlaybackRate(self.speed)
            if self.player.Tell() == 0:
                self.OnPlay()
            else:
                value = self.player.Tell() - self.player.starttime
                self.sldSeek.SetValue(value)
                if value > self.player.duration:
                    self.OnStop(None)
    
    def OnPlay(self, event=None):
        if self.playerState == True:
            self.mainPanel.Enable()
            self.cursor = wx.NullCursor
            self.info = None
            if not self.player.Play():
                wx.MessageBox("Unable to Play media %s: Unsupported format?" % self.player.filepath,
                          "ERROR",
                          wx.ICON_ERROR | wx.OK)
            else:
                self.player.SetInitialSize()
                self.player.SeekToStart()
                self.mainPanel.GetSizer().Layout()
                self.sldSeek.SetRange(0, self.player.duration)


    def OnPause(self, event):
        if self.playerState == True:
            self.player.Pause()

    def OnStop(self, event=None):
        if self.playerState == True:
            self.player.SeekToStart()
            self.player.Pause()

    def OnSeek(self, event):
        value = self.sldSeek.GetValue() + self.currentData.starttime
        self.statusBar.SetStatusText(str(value), 0)
        self.player.Seek(value)


    def OnQuitApplication(self, event):
        self.preferences.SavePreferences()
        self.frame.Destroy()
                
    def OnAboutApplication(self, event):
        aboutFrame = self.res.LoadFrame(None, 'aboutFrame')
        aboutFrame.Show()
                
    def OnHelpApplication(self, event):
        helpFrame = self.res.LoadFrame(None, 'helpFrame')
        helpFrame.Show()
                
    def OnVolumeChange(self, event):
        self.volume = self.sldVolume.GetValue() / 100.0
        self.statusBar.SetStatusText("Volume change to %s" % str(self.volume))
        self.player.SetVolume(self.volume)
        
    def OnLeftAddition(self, event):
        self.leftAddition = self.sldLeftAddition.GetValue()
        self.statusBar.SetStatusText("Added %s msec at the beginning" % str(self.leftAddition))
        self.player.InitInfo(self.currentData.starttime, self.currentData.duration, self.leftAddition, self.rightAddition)
        
    def OnRightAddition(self, event):
        self.rightAddition = self.sldRightAddition.GetValue()
        self.statusBar.SetStatusText("Added %s msec at the end" % str(self.rightAddition))
        self.player.InitInfo(self.currentData.starttime, self.currentData.duration, self.leftAddition, self.rightAddition)
        
    def OnSpeedChange(self, event):
        self.speed = self.sldSpeed.GetValue() / 100.0
        self.statusBar.SetStatusText("Playback rate change to %s" % str(self.speed))
        self.player.SetPlaybackRate(self.speed)  

    def OnOriginalVideoFolderSelect(self, event):
        dlg = wx.DirDialog(self.frame, "Choose a directory:",
                  style=wx.DD_DEFAULT_STYLE )
        if dlg.ShowModal() == wx.ID_OK:
            self.preferences.originalVideosFolder = dlg.GetPath() + os.sep
        dlg.Destroy()


    def OnSegmentedVideoFolderSelect(self, event):
        dlg = wx.DirDialog(self.frame, "Choose a directory:",
                  style=wx.DD_DEFAULT_STYLE )
        if dlg.ShowModal() == wx.ID_OK:
            self.preferences.segmentedVideosFolder = dlg.GetPath() + os.sep
        dlg.Destroy()
    
    def OnUseLocalVideosChecked(self, event):
        self.preferences.useLocalVideos = self.chkBoxUseLocalVideos.GetValue()
 
    def OnShowSegmentedChecked(self, event):
        self.preferences.showSegmented = self.chkBoxShowSegmented.GetValue()
 
    def OnSearchTextChanged(self, event):
        value = self.searchCtrl.GetValue()
        for i,v in enumerate(value):
            if not MFCD_Functions.IsAllowedCharacter(v):
                self.searchCtrl.SetValue(value.replace(v,""))
                self.searchCtrl.SetInsertionPoint(i)
                self.statusBar.SetStatusText("Only characters in Turkish alphabet are allowed!")
                break
    def OnDoSearch(self, evt):
        value = self.searchCtrl.GetValue()
        if value == "":
            self.statusBar.SetStatusText("Empty search query!")
        else:
            self.statusBar.SetStatusText("Searching: " + value)
            self.Search(value)
            self.searchCtrl.SetValue("")  # make the search box empty
    
    def OnRecentSearch(self, event):
        self.statusBar.SetStatusText("Searched: " + self.recentSearches[event.Id])
        self.Search(self.recentSearches[event.Id])

    def Search(self, value):
        searchText = MFCD_Functions.Tr2eng(value)
        if MFCD_Functions.Eng2tr(searchText) in self.recentSearches:
            self.FillTree(searchText)
        else:
            if self.client.IsServiceAvailable():
                self.info = wx.BusyInfo("Searching. Please wait...")
                self.mainPanel.Disable()
                tmp = wx.BusyCursor()
                result = self.client.SendQuery(searchText)
                f = open(self.searchResultsFolder + searchText + ".txt", "w")
                f.write(result)
                f.close()
                self.FillTree(searchText)
                self.mainPanel.Enable()
                self.info = None
            else:
                wx.MessageBox("Service is not running or there is a problem in your internet connection.\nYou can only search in history.",
                              "Service Unavailable",
                          wx.ICON_ERROR | wx.OK)
        self.ResetSearchHistoryMenu()
    
    def FillTree(self, searchText): 
        
        self.searchText = searchText
        filepath = self.searchResultsFolder + searchText + '.txt'
        if MFCD_Reader.IsEmptyResult(filepath):
            wx.MessageBox("Sorry, no results returned for %s " % MFCD_Functions.Eng2tr(searchText), "No results!",
                          wx.ICON_WARNING | wx.OK)
        else:
            self.reader = MFCD_Reader.MFCD_Reader(filepath)
            distinctElements = MFCD_Functions.Distinct([self.reader.data[x][0] for x in range(self.reader.numOfFiles)])
            
            self.treectrlSearch.DeleteAllItems()
            
            self.frame.rootnode = self.treectrlSearch.AddRoot(" ")
            for distinct in distinctElements:
                child = self.treectrlSearch.AppendItem(self.frame.rootnode, MFCD_Functions.Filename2Date(distinct))    
                for data in self.reader.data:
                    if data[0] == distinct:
                        item = self.treectrlSearch.AppendItem(child,  MFCD_Functions.msec2Time(data[1]) + " " + MFCD_Functions.Value2Stars(data[3]))
                self.treectrlSearch.Expand(child)

        
    def MakeSearchHistoryMenu(self):
        menu = wx.Menu()
        self.recentSearches = MFCD_Functions.GetFileNamesInDir(self.searchResultsFolder)        
        if len(self.recentSearches) == 0:
            item = menu.Append(-1, "No recent search")
        else:
            item = menu.Append(-1, "Clear History")
            self.frame.Bind(wx.EVT_MENU, self.OnClearHistory, item)
            menu.AppendSeparator()
            item = menu.Append(-1, "Search History")
        item.Enable(False)
        
        for i, txt in enumerate(self.recentSearches):
            item = menu.Append(i, txt)
            self.frame.Bind(wx.EVT_MENU, self.OnRecentSearch, item)

        return menu
    
    def ResetSearchHistoryMenu(self):
        self.searchCtrl.SetMenu(self.MakeSearchHistoryMenu())

    def OnClearHistory(self, event):
        dlg = wx.MessageDialog(self.frame, 'Are you sure to delete the search history!',
                       'Warning',
                       wx.YES_NO | wx.ICON_WARNING
                       )
        id = dlg.ShowModal()
        if ( id == wx.ID_YES):
            MFCD_Functions.DelFilesInDir(self.searchResultsFolder)
            self.ResetSearchHistoryMenu()
            self.statusBar.SetStatusText("Search history cleared!")
        else:
            self.statusBar.SetStatusText("Canceled by user")
        dlg.Destroy()
    
    def OnSelChanged(self, event):
        item = event.GetItem()
        if self.treectrlSearch.GetRootItem() == self.treectrlSearch.GetItemParent(self.treectrlSearch.GetItemParent(item)):
            startTime = MFCD_Functions.Time2msec(self.treectrlSearch.GetItemText(item))
            for i, d in enumerate(self.reader.data):
                if d[1] == startTime: break
            self.currentData.filename = self.reader.data[i][0]
            self.currentData.starttime = self.reader.data[i][1]
            self.currentData.duration = self.reader.data[i][2]
            self.currentData.relevance = self.reader.data[i][3]
            
            self.FillProperties()
            self.LoadVideo(self.currentData.filename)
    
    def FillProperties(self):
        date = MFCD_Functions.Filename2Date(self.currentData.filename)
        propertiesStr = "Query: " + MFCD_Functions.Eng2tr(self.searchText) + "\n" + \
                        "Date: " + date[0:10] + "\n" + \
                        "Time: " + date[12:17] + "\n" + \
                        "Start Time: " + MFCD_Functions.msec2Time(self.currentData.starttime) + "\n" + \
                        "Duration: " + MFCD_Functions.msec2Time(self.currentData.duration) + "\n" + \
                        "Relevance: " + str(self.currentData.relevance) + ""
        self.stxtProperties.SetLabel(propertiesStr)


if __name__ == '__main__':
    app = MFCD(False)
    app.MainLoop()